local _, _, REVISION = string.find("$Revision: 17552 $", "(%d+)")
local lang = AceLibrary("AceLocale-2.2"):new("SchoolFU")
local dewdrop = AceLibrary("Dewdrop-2.0")
local abacus = AceLibrary("Abacus-2.0")
local crayon = AceLibrary("Crayon-2.0")
local tablet = AceLibrary("Tablet-2.0")
-- local toolbox = AceLibrary("Toolbox-1.0")
-- local deformat = AceLibrary("Deformat-2.0")

lang:RegisterTranslations("enUS", function() return {
	["FuBar"] = true,

	["Show"] = true,
	["Show school data."] = true,
    
	["Advanced Mode"] = true,
	["Toggles advanced school options visibility."] = true,    
} end)

SchoolFU = School:NewModule("FuBar", "FuBarPlugin-2.0")

SchoolFU:SetDebugging(false)
SchoolFU:SetDebugLevel(3)

SchoolFU.revision = tonumber(REVISION)

SchoolFU.title = "School"
SchoolFU.hasIcon = "Interface\\Icons\\INV_Misc_Book_08"
SchoolFU.clickableTooltip = false;
SchoolFU.cannotDetachTooltip = true; 
SchoolFU.hasNoText = true

SchoolFU.defaults = {
    forceRebuild = false,
	advancedMode = false,
}


SchoolFU.OnMenuRequest = {
                          type = 'group',
                          args = {}
}

if (SchoolUI) then
    -- Lock Position
    SchoolFU.OnMenuRequest.args.lock = {
        type = "toggle",
        name = "Lock position",
        desc = "Toggles locking the School frame in place.",
        get = function() return SchoolUI.db.profile.locked end,
        set = function(v) SchoolUI.db.profile.locked = v end,
        order = 120,
    }
    
    SchoolFU.OnMenuRequest.args.fixedtooltip = {
        type = "toggle",
        name = "Fixed Tooltip",
        desc = "Locks the tooltip in place on the left side of the frame.",
        get = function() return SchoolUI.db.profile.fixedTooltip end,
        set = function(v) SchoolUI.db.profile.fixedTooltip = v end,
        order = 130,
    }
    
    SchoolFU.OnMenuRequest.args.tooltipposition = {
        type = "text",
        name = "Tooltip Position",
        desc = "Sets tooltip placement on left or right of frame.",
        get = function() return SchoolUI.db.profile.tooltipPosition end,
        set = function(v) SchoolUI.db.profile.tooltipPosition = v end,
        validate = {
            ["left"] = "Left",
            ["right"] = "Right",
        },
        order = 140,
    }
    
    SchoolFU.OnMenuRequest.args.detailposition = {
        type = "text",
        name = "Detail Position",
        desc = "Sets detail placement on left or right of frame.",
        get = function() return SchoolUI.db.profile.detailPosition end,
        set = function(v) SchoolUI.db.profile.detailPosition = v end,
        validate = {
            ["left"] = "Left",
            ["right"] = "Right",
        },
        order = 150,
    }
    
    SchoolFU.OnMenuRequest.args.rows = {
        type = "range",
        name = "Rows",
        desc = "Sets result row count.\n|cffff0000REQUIRES RELOADUI|r",
        min = 5,
        max = 25,
        step = 1,
        isPercent = false,
        get = function() return SchoolUI.db.profile.resultRows end,
        set = function(v) SchoolUI.db.profile.resultRows = v end,
        order = 160,
    }
    
    SchoolFU.OnMenuRequest.args.backgroundalpha = {
        type = "range",
        name = "Backdrop Alpha",
        desc = "Sets frame background transparency level.",
        min = 0,
        max = 1,
        step = 0,
        isPercent = false,
        get = function() return SchoolUI.db.profile.backgroundAlpha end,
        set = function(v)
            SchoolUI.db.profile.backgroundAlpha = v
            SchoolUI:SetBackdropAlpha()
        end,
        order = 170,
    }
    
    SchoolFU.OnMenuRequest.args.ignorecost = {
        type = "toggle",
        name = "Ignore cost",
        desc = "Ignore learning cost.",
        get = function() return SchoolUI.db.profile.ignoreCost end,
        set = function(v) 
            SchoolUI.db.profile.ignoreCost = v 
            SchoolUI.UpdateResults()
        end,
        order = 180,
    }

    SchoolFU.OnMenuRequest.args.ignorecombat = {
        type = "toggle",
        name = "Ignore combat",
        desc = "Ignore combat.",
        get = function() return SchoolUI.db.profile.ignoreCombat end,
        set = function(v) SchoolUI.db.profile.ignoreCombat = v end,
        order = 190,
    }
else
    SchoolFU.OnMenuRequest.args.ignorecost = {
        type = "toggle",
        name = "Ignore cost",
        desc = "Ignore learning cost.",
        get = function() return SchoolFU.db.profile.ignoreCost end,
        set = function(v) 
            SchoolFU.db.profile.ignoreCost = v 
            SchoolFU.UpdateResults()
        end,
        order = 180,
    }

    SchoolFU.OnMenuRequest.args.ignorecombat = {
        type = "toggle",
        name = "Ignore combat",
        desc = "Ignore combat.",
        get = function() return SchoolFU.db.profile.ignoreCombat end,
        set = function(v) SchoolFU.db.profile.ignoreCombat = v end,
        order = 190,
    }

end

--[[
-- using an AceOptions data table
SchoolFU.OnMenuRequest = {
                           type = 'group',
                           args = {
                                   forceRebuild = {
                                                   name = "Force rebuild",
                                                   desc = "Force a rebuild of available skill.",
                                                   type = "toggle",
                                                   get = "IsForceRebuild",
                                                   set = "ToggleForceRebuild",
                                                  }
                                  }
                           }
--]]

function SchoolFU:OnTooltipUpdate()
    self:CreateUI()
end

function SchoolFU:OnClick()
    self:OpenUI()
end

function SchoolFU:OpenUI()
    if (SchoolUI) then
        if (not SchoolUI:IsShown()) then
            SchoolUI:ShowUI()
        else
            SchoolUI:HideUI()
        end
    else 
        self:CreateUI()
    end
end

function SchoolFU:FetchResults()
    local school = School:GotoSchool()
    if (not school) then return end

    local show = {}
    local money = GetMoney()
    local spells = school["spells"]
    local trainers = school["trainers"]

    if (spells ~= nil) then
        for k,v in pairs(spells) do
            if (v ~= nil) then
                -- self:LevelDebug(3, "Key ==> %s", k)
                for vk, vv in pairs(v) do
                    -- self:LevelDebug(3, "Value Key ==> %s", vk) 
                    if (vv ~= nil) then
                        local spell = vv

                        if (trainers ~= nil) then
                            self:LevelDebug(3, "Found trainers.")
                            if (trainers[spell.name] ~= nil) then
                                self:LevelDebug(3, "Found trainers for " .. spell.name)
                                if (trainers[spell.name][spell.subtext] ~= nil) then
                                    self:LevelDebug(3, "Found trainers for " .. spell.name .. " Rank: " .. spell.subtext)
                                    if (not spell.trainers) then spell.trainers = {} end
                                    for tk, tv in pairs(trainers[spell.name][spell.subtext]) do
                                        if (tv ~= nil) then
                                            table.insert(spell.trainers, tv)
                                        end
                                    end
                                end
                            end
                        end
                        
                        local ignoreCost = false
                        if (not SchoolUI) then
                            ignoreCost = self.db.profile.ignoreCost
                        else
                            ignoreCost = SchoolUI.db.profile.ignoreCost
                        end
                        
                        if (not ignoreCost) then
                            if (spell.cost.money ~= nil) and (money >= spell.cost.money) then table.insert(show, spell) end
                        else
                            table.insert(show, spell)
                        end
                    end
                end
            end
        end
    end
    
    --[[
    for k,v in pairs(show) do
        if (v ~= nil) then
            for vk, vv in pairs(v) do
                if (vv ~= nil) then
                    self:Debug("Value Key ==> ", vk)
                end
            end
        end
    end
    --]]

    local results = {}      
    results.count = 0
    if (show ~= nil) then
        for k,v in pairs(show) do
            if (v ~= nil) then
                results.count = results.count + 1
                local result = v
                if (not results[result.category]) then results[result.category] = {} end
                if (not results[result.category][result.fullname]) then results[result.category][result.fullname] = {} end
                results[result.category][result.fullname] = result
            end
        end
    end
    
    return results
end

function SchoolFU:CreateUI()
    self:UpdateResults()
    if (SchoolUI) then
        tablet:SetHint("Click to open/close School UI.")
    end
end

function SchoolFU:UpdateResults()
    self.schoolResults = self:FetchResults()
    if (not self.schoolResults) then return end
    if (not self.schoolResults.count) or (self.schoolResults.count < 1) then return end
    
    self.studentMoney = GetMoney()
    self.numResults = self.schoolResults.count
    
    for k,v in pairs(self.schoolResults) do
        if (v ~= nil) then
            -- self:Debug("Key ==> %s", k)
            if (type(v) == "table") then
                local cat = tablet:AddCategory(
                    'id', k,
                    'text', k,
                    'columns', 4,
                    'child_textR', 1,
                    'child_textG', 1,
                    'child_textB', 0,
                    'child_textR2', 1,
                    'child_textG2', 1,
                    'child_textB2', 1,
                    'child_justify3', "LEFT"
                )
                
                for vk, vv in pairs(v) do
                    if (vv ~= nil) then
                        local resultline = {}
                        local result = vv
                        
                        if (not result.cost.formatted) then result.cost.formatted = {} end
                        if (not result.cost.formatted.money) then result.cost.formatted.money = abacus:FormatMoneyFull(result.cost.money, true, true) end

                        cat:AddLine(
                            'text', result.fullname,
                            'text2', "     ",
                            'text3', "     ", -- result.icon
                            'text4', result.cost.formatted.money
                        )
                        
                        if (result.trainers ~= nil) then
                            for rtk, rtv in pairs(result.trainers) do
                                if (rtv ~= nil) then
                                    -- self:Debug("rtk ==> %s", rtk)
                                    -- self:Debug("type(rtv) ==> %s", type(rtv))
                                    local trainer = rtv
                                    if (not trainer.formatted) then trainer.formatted = {} end
                                    trainer.formatted.name = crayon:Green(trainer.name)
                                    trainer.formatted.type = trainer.type
                                    if (trainer.zone ~= nil) then trainer.formatted.location = trainer.zone end
                                    if (trainer.subzone ~= nil) then 
                                        trainer.formatted.location = trainer.subzone .. "(" .. trainer.formatted.location .. ")"
                                    end
                                    cat:AddLine(
                                        'text', trainer.formatted.name,
                                        'text2', "     ",
                                        'text3', trainer.formatted.type,
                                        'text4', trainer.formatted.location
                                    )
                                end
                            end
                        end
                    end
                end
            end
        end
    end

    if (not SchoolFU.hasNoText) then self:SetText(string.format("New skills: %d", self.numResults)) end
end


