local _, _, REVISION = string.find("$Revision: 17552 $", "(%d+)")
local lang = AceLibrary("AceLocale-2.2"):new("SchoolStudent")

lang:RegisterTranslations("enUS", function() return {
	["Student"] = true,

	["Rebuild"] = true,
	["Rebuild student data."] = true,
    
	["Advanced Mode"] = true,
	["Toggles advanced student options visibility."] = true,    
} end)

SchoolStudent = School:NewModule("Student")


SchoolStudent.defaults = {
	advancedMode = false,
}

SchoolStudent.options = {
    rebuild = {
               type = 'execute',
               name = lang["Rebuild"],
               desc = lang["Rebuild student data."],
               func = function() SchoolStudent:Rebuild() end,
               order = 110,
    },

	advancedmode = {
		type = "toggle",
		name = lang["Advanced Mode"],
		desc = lang["Toggles advanced student options visibility."],
		get = function() return SchoolStudent:ToggleAdvancedMode() end,
		set = function(v) SchoolStudent:ToggleAdvancedMode(v) end,
		order = 120,
	},
}



SchoolStudent:SetDebugging(false)
SchoolStudent:SetDebugLevel(3)

SchoolStudent.revision = tonumber(REVISION)
SchoolStudent.triggers = true

function SchoolStudent:OnInitialize()
end

function SchoolStudent:OnEnable()
    self:RegisterEvent("CHARACTER_POINTS_CHANGED")
    self:RegisterEvent("PLAYER_LEVEL_UP")
    self:RegisterEvent("CHAT_MSG_SKILL")
    self:RegisterEvent("SPELLS_CHANGED")
    
    self.db.char.level = UnitLevel("player")
    local cp1, cp2 = UnitCharacterPoints("player");
    self.db.char.charpoints = {}
    self.db.char.charpoints.cp1 = cp1
    self.db.char.charpoints.cp2 = cp2

end

function SchoolStudent:OnDisable()
end

function SchoolStudent:ToggleAdvancedMode()
	if type(value) == "nil" then
		return self.db.profile.advancedMode
	end

	self.db.profile.advancedMode = value
end

function SchoolStudent:GetData()
    local data = {}
    
    if (not self.db.char.level) then self:LogPlayerLevel() end
    data.level = self.db.char.level
    
    if (not self.db.char.money) then self:LogPlayerMoney() end
    data.money = self.db.char.money
    
    if (not self.db.char.charpoints) then self:LogCharacterPoints() end
    data.charpoints = self.db.char.charpoints
    
    if (not self.db.char.spells) then self:LogSpellInfo() end
    data.spells = self.db.char.spells
    
    if (not self.db.char.talents) then self:LogTalentInfo() end
    data.talents = self.db.char.talents
    
    if (not self.db.char.tradeskills) then self:LogTradeskillInfo() end
    data.tradeskills = self.db.char.tradeskills
    
    if (not self.db.char.skills) then self:LogSkillInfo() end
    data.skills = self.db.char.skills
    
    if (not self.db.char.professions) then self:LogProfessions() end
    data.professions = self.db.char.professions
    
    return data
end

function SchoolStudent:CHARACTER_POINTS_CHANGED()
    self:LogCharacterPoints()
    self:LogTalentInfo()
end

function SchoolStudent:PLAYER_LEVEL_UP()
    self:LogPlayerLevel()
end

function SchoolStudent:CHAT_MSG_SKILL()
    self:LogSkillInfo()
    self:LogProfessions()
end

function SchoolStudent:SPELLS_CHANGED()
    self:LogSpellInfo()
end

function SchoolStudent:Rebuild()
    self.db.char.level = nil
    self:LogPlayerLevel()

    self.db.char.money = nil
    self:LogPlayerMoney()

    self.db.char.charpoints = nil
    self:LogCharacterPoints()

    self.db.char.spells = nil
    self:LogSpellInfo()
    
    self.db.char.talents = nil
    self:LogTalentInfo()
    
    self.db.char.tradeskills = nil
    self:LogTradeskillInfo()
    
    self.db.char.skills = nil
    self:LogSkillInfo()
    
    self.db.char.professions = nil
    self:LogProfessions()
    
    return self:GetData()
end

function SchoolStudent:LogCharacterPoints()
    local cp1, cp2 = UnitCharacterPoints("player");
    self.db.char.charpoints = {}
    self.db.char.charpoints.cp1 = cp1
    self.db.char.charpoints.cp2 = cp2
end

function SchoolStudent:LogPlayerLevel()
    self.db.char.level = UnitLevel("player")
end

function SchoolStudent:LogPlayerMoney()
    self.db.char.money = GetMoney()
end

function SchoolStudent:LogSpellInfo()
    local spells = {}

    local i = 1
    while true do
        local spell = {}
        
        spell.name, spell.rank = GetSpellName(i, BOOKTYPE_SPELL)
        if (not spell.name) then
            do break end
        end
        
        spell.icon = GetSpellTexture(i, BOOKTYPE_SPELL)
        spell.passive = IsPassiveSpell(i, BOOKTYPE_SPELL)
        
        if (not spells["player"]) then spells["player"] = {} end
        if (not spells["player"][i]) then spells["player"][i] = {} end
        
        spells["player"][i] = spell
       
        i = i + 1
    end
    
    local j = 1
    while true do
        local spell = {}
        
        spell.name, spell.rank = GetSpellName(j, BOOKTYPE_PET)
        if (not spell.name) then
            do break end
        end
        
        spell.icon = GetSpellTexture(j, BOOKTYPE_PET)
        spell.passive = IsPassiveSpell(j, BOOKTYPE_PET)
        
        if (not spells["pet"]) then spells["pet"] = {} end
        if (not spells["pet"][j]) then spells["pet"][j] = {} end
        
        spells["pet"][j] = spell
    
        j = j + 1
    end
    
    self.db.char.spells = spells
end

function SchoolStudent:LogTalentInfo()
    local talents = {}
    
    for t = 1, GetNumTalentTabs() do
        local numTalents = GetNumTalents(t)
    
        for i = 1, numTalents do
            nameTalent, icon, tier, column, currRank, maxRank = GetTalentInfo(t, i)
            
            if (nameTalent ~= nil) then
                local talent = {}
                
                talent.name = nameTalent
                talent.icon = icon
                talent.tier = tier
                talent.column = column
                talent.rank = currRank
                talent.maxrank = maxRank
                
                talents[talent.name] = talent
            end
        end
    end
    
    self.db.char.talents = talents
end

function SchoolStudent:LogSkillInfo()
    local skills = {}
    
    for i = 1, GetNumSkillLines() do
        local skillName, header, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank, isAbandonable, stepCost, rankCost, minLevel, skillCostType = GetSkillLineInfo(i);

        if (skillName ~= nil) and (not header) then
            local skill = {}
            skill.name = skillName
            skill.rank = skillRank
            skill.maxrank = skillMaxRank
            skill.modifier = skillModifier
            skill.abandon = isAbandonable
            skill.level = minLevel
            skill.cost = {}
            skill.cost.step = stepCost
            skill.cost.rank = rankCost
            skill.cost.type = skillCostType
            
            skills[skill.name] = skill
        end
    end

    self.db.char.skills = skills

    --[[
    for k,v in pairs(self.db.char.skills) do
        if (v ~= nil) then
            self:Debug("Key ==> %s\nValue ==> %s", k, v)
        end
    end
    --]]
end

function SchoolStudent:LogProfessions()

    local professions = {}
    
    if (not self.db.char.skills) then self:LogSkillInfo() end
    local skills = self.db.char.skills
    
    -- Alchemy
    -- Blacksmithing
    -- Cooking
    -- Enchanting
    -- Engineering
    -- First Aid
    -- Fishing
    -- Herbalism
    -- Jewelcrafting
    -- Leatherworking
    -- Mining
    -- Skinning
    -- Tailoring 

    local listProfessions = {}
    listProfessions.primary = {
                               ["Alchemy"] = true,
                               ["Blacksmithing"] = true,
                               ["Enchanting"] = true,
                               ["Enginering"] = true,
                               ["Herbalism"] = true,
                               ["Jewelcrafting"] = true,
                               ["Leatherworking"] = true,
                               ["Mining"] = true,
                               ["Skinning"] = true,
                               ["Tailoring"] = true,
    }
    listProfessions.secondary = {
                                 ["Cooking"] = true,
                                 ["First Aid"] = true,
                                 ["Fishing"] = true,
    }
    
                           
                             
    if (self.db.char.professions ~= nil) then professions = self.db.char.professions end
    
    for k,v in pairs (skills) do
        if (v ~= nil) then
            local skill = v
            for lpk, lpv in pairs(listProfessions) do
                if (not professions[lpk]) then professions[lpk] = {} end
                if (not professions[lpk].count) then professions[lpk].count = 0 end                
                if (not professions[lpk][skill.name]) then 
                    if (lpv[skill.name] ~= nil) then
                        professions[lpk][skill.name] = {}
                        professions[lpk][skill.name].max = skill.maxrank
                        professions[lpk][skill.name].rank = skill.rank
                        professions[lpk].count = professions[lpk].count + 1
                    end
                else
                    professions[lpk][skill.name].max = skill.maxrank
                    professions[lpk][skill.name].rank = skill.rank
                end
            end
        end
    end

    self.db.char.professions = professions

    --[[
    for k,v in pairs(self.db.char.professions) do
        if (v ~= nil) then
            for vk, vv in pairs(v) do
                if (vv ~= nil) then
                    self:Debug("Value Key ==> %s", vk)
                end
            end
        end
    end
    --]]
end

function SchoolStudent:LogTradeskillInfo()
    -- self:LevelDebug(3, "LogTradeskillInfo")
    
    
    --[[
    local lNumTradeSkills = GetNumTradeSkills();
    local lTradeSkillCategory = '';
    local lTradeSkillLineArray;
    local lTradeSkillInfo;
    
    
    --[=[
    -- If the Character doesn't have any Trade Skills, there is no need to continue
    if ( lNumTradeSkills == 0 ) then
        p("No tradeskills.")
        return;
    else
        p(lNumTradeSkills.." tradeskills found.")
    end    
    --]=]
    if (lNumTradeSkills == 0) then return end
    
    for idx = 1, lNumTradeSkills,  1 do
        local lTradeSkillName, lTradeSkillType, lNumAvailable, lIsExpanded = GetTradeSkillInfo(idx);
        
        if (lTradeSkillName ~= nil) then
            if ( lTradeSkillType == "header" ) then
                lTradeSkillCategory = lTradeSkillName;
            else
                SelectTradeSkill(idx);
                
                local lTradeSkillLineName, lTradeSkillLineRank, lTradeSkillLineMaxRank = GetTradeSkillLine();
                local lTradeSkillMinMade, lTradeSkillMaxMade = GetTradeSkillNumMade(idx);
                local lTradeSkillIcon = GetTradeSkillIcon(idx);
                local lTradeSkillResultItemLink = GetTradeSkillItemLink(idx);
                local lTradeSkillResult = Teacher:AddBaseItemInfo(lTradeSkillResultItemLink, lTradeSkillIcon);
                
                if not (db_Professions[lTradeSkillLineName]) then
                    db_Professions[lTradeSkillLineName] = {};
                end
                
                lTradeSkillLineArray = db_Professions[lTradeSkillLineName];
                
                if not (lTradeSkillLineArray[lTradeSkillName]) then
                    lTradeSkillLineArray[lTradeSkillName] = {};
                end
                
                lTradeSkillInfo = lTradeSkillLineArray[lTradeSkillName];
                
                if (lTradeSkillInfo["Category" ] ~= lTradeSkillCategory) then
                    lTradeSkillInfo["Category"] = lTradeSkillCategory;
                end     
                if (lTradeSkillInfo["Icon"] ~= lTradeSkillIcon ) then
                    lTradeSkillInfo["Icon"] = lTradeSkillIcon;
                end     
                if (lTradeSkillMinMade ~= nil) then
                    lTradeSkillInfo["MinMade"] = lTradeSkillMinMade
                end
                if (lTradeSkillMaxMade ~= nil) then
                    lTradeSkillInfo["MaxMade"] = lTradeSkillMaxMade
                end
                
                if (lTradeSkillLineRank ~= nil) then
                    lTradeSkillInfo["Rank"] = lTradeSkillLineRank
                end
                if (lTradeSkillLineMaxRank ~= nil) then
                    lTradeSkillInfo["MaxRank"] = lTradeSkillLineMaxRank
                end
                
                if (lTradeSkillResult ~= nil) then
                    lTradeSkillInfo["Result"] = lTradeSkillResult["Item"] .. ":" .. lTradeSkillResult["Name"];
                end
            end
        end
    end


    --]]
    self.db.char.skills = {}
end
