local _, _, REVISION = string.find("$Revision: 17552 $", "(%d+)")
local lang = AceLibrary("AceLocale-2.2"):new("SchoolTeacher")
local tooltip = AceLibrary("Gratuity-2.0")

lang:RegisterTranslations("enUS", function() return {
	["Teacher"] = true,

	["Rebuild"] = true,
	["Rebuild teacher data."] = true,
    
    ["Take Notes"] = true,
    ["Toggle note taking."] = true,
} end)

SchoolTeacher = School:NewModule("Teacher")

SchoolTeacher.defaults = {
	advancedMode = false,
    takeNotes = true,
}

SchoolTeacher.options = {
    rebuild = {
               type = 'execute',
               name = lang["Rebuild"],
               desc = lang["Rebuild teacher data."],
               func = function() SchoolTeacher:Rebuild() end,
               order = 110,
    },

    takenotes = {
        type = "toggle",
        name = lang["Take Notes"],
        desc = lang["Toggle note taking."],
        get = function() return SchoolTeacher.db.profile.takeNotes end,
        set = function(v) SchoolTeacher.db.profile.takeNotes = v end,
        order = 130,
    },
}

SchoolTeacher:SetDebugging(false)
SchoolTeacher:SetDebugLevel(3)

SchoolTeacher.revision = tonumber(REVISION)

function SchoolTeacher:OnInitialize()
end

function SchoolTeacher:OnEnable()
    self:RegisterEvent("TRAINER_SHOW")
    self:RegisterEvent("TRAINER_UPDATE")
    self:RegisterEvent("TRAINER_CLOSED")
    
    self:RegisterEvent("MERCHANT_SHOW")
    self:RegisterEvent("MERCHANT_UPDATE")
    self:RegisterEvent("MERCHANT_CLOSED")
    
    self.class = self.core.classid
end

function SchoolTeacher:OnDisable()
end

function SchoolTeacher:GetData()
    local data = {}

    data.spells = self.db.char.spells
    data.trainers = self.db.char.trainers
    
    return data
end

function SchoolTeacher:TRAINER_SHOW()
    -- self:Debug("TRAINER_SHOW")
    self.IsAtTrainer = true
end

function SchoolTeacher:TRAINER_UPDATE()
    -- self:Debug("TRAINER_UPDATE")
end

function SchoolTeacher:TRAINER_CLOSED()
    -- self:Debug("TRAINER_CLOSED")
    if (self.IsAtTrainer) then
        -- self:Debug("self.IsAtTrainer ==> %s", self.IsAtTrainer)
        self:LogTrainerInfo()
        self.IsAtTrainer = false
    end
end

function SchoolTeacher:MERCHANT_SHOW()
    -- self:Debug("MERCHANT_SHOW")
    self.IsAtMerchant = true
    self:LogMerchantInfo()
end

function SchoolTeacher:MERCHANT_UPDATE()
    -- self:Debug("MERCHANT_UPDATE")
end

function SchoolTeacher:MERCHANT_CLOSED()
    -- self:Debug("MERCHANT_CLOSED")
    if (self.IsAtMerchant) then
        -- self:LogMerchantInfo()
        self.IsAtMerchant = false
    end
end

function SchoolTeacher:Rebuild()
    if (self.IsAtTrainer) or (self.IsAtMerchant) then
        self.db.char.spells = nil
        self.db.char.trainers = nil
    end
    
    if (self.IsAtTrainer) then
        self:LogTrainerInfo()
    end    
    
    if (self.IsAtMerchant) then
        self:LogMerchantInfo()
    end
    
    return self:GetData()
end

function SchoolTeacher:LogTrainerInfo()
    if (self.IsAtTrainer) then
        self:LogClassTrainerInfo()
    end
    
    if (self.db.char.spells ~= nil) then
        self:TriggerEvent("School_Teacher_Changed")
    end
    
end
function SchoolTeacher:LogMerchantInfo()
    if (self.IsAtMerchant) then
        self:LogClassMerchantInfo()
    end
    
    if (self.db.char.spells ~= nil) then
        self:TriggerEvent("School_Teacher_Changed")
    end
end

function SchoolTeacher:LogTradeskillTrainerInfo()
    self:LevelDebug(3, "LogTradeskillTrainerInfo")
end

function SchoolTeacher:LogClassTrainerInfo()
    self:Debug("LogClassTrainerInfo")
    local spells = {}
    local trainers = {}
    local notes = {}

    if (self.db.char.spells ~= nil) then spells = self.db.char.spells end
    if (self.db.char.trainers ~= nil) then trainers = self.db.char.trainers end
    if (self.db.char.notes ~= nil) then notes = self.db.char.notes end
    
    local npc = {}
    npc.name = self:GetTeacherName()
    npc.zone = self:GetTeacherZone()
    npc.subzone = self:GetTeacherSubZone()
    npc.type = self:GetTeacherType()
    
    local filter = {}
    filter.available = GetTrainerServiceTypeFilter("available")
    filter.unavailable = GetTrainerServiceTypeFilter("unavailable")
    filter.used = GetTrainerServiceTypeFilter("used")
    
    SetTrainerServiceTypeFilter("available", 1)
    SetTrainerServiceTypeFilter("unavailable", 1)
    SetTrainerServiceTypeFilter("used", 1)
    
    ExpandTrainerSkillLine(0)
    
    local numTrainerServices = GetNumTrainerServices()
    
    if (not numTrainerServices or (numTrainerServices < 1)) then
        return
    end
    
    local category = nil
    for i = 1, numTrainerServices do
        local service = {}
        
        service.name, service.subtext, service.type, service.expanded = GetTrainerServiceInfo(i)

        if (not service.name or not service.type) then return end
        if (service.type == "header") then
            category = service.name
        else
            if (category ~= nil) then service.category = category end
            if (not service.subtext) or (string.len(service.subtext) < 1) then service.subtext = "None" end
            service.fullname = service.name
            
            if (service.subtext ~= "None") then
                -- Check for "RANK"
                service.fullname = service.fullname .. " (" .. service.subtext .. ")"
            end
            
            service.cost = {}
            -- Money, Character Point Cost 1, Character Point Cost 2
            service.cost.money, service.cost.cost1, service.cost.cost2 = GetTrainerServiceCost(i)

            service.icon = GetTrainerServiceIcon(i)
            service.level = GetTrainerServiceLevelReq(i)
        
            local req = {}
            req.skill, req.rank, req.hasReq = GetTrainerServiceSkillReq(i)
            if (req.skill) then service.req = req end
            
            local numRequirements = GetTrainerServiceNumAbilityReq(i);
            if (numRequirements > 0) then
                service.ability = {}
                for k = 1, numRequirements do
                    service.ability[k] = {}
                    service.ability[k].skill, service.ability[k].req = GetTrainerServiceAbilityReq(i, k)
                end
            end

            -- Fix wrapped descriptions
            -- service.description = GetTrainerServiceDescription(i)
            
            service.learnspell, service.petlearnspell = IsTrainerServiceLearnSpell(i)
            
            
            if (not spells[service.category]) then spells[service.category] = {} end
            if (not spells[service.category][service.name]) then spells[service.category][service.name] = {} end
            if (not spells[service.category][service.name][service.subtext]) then spells[service.category][service.name][service.subtext] = {} end
            spells[service.category][service.name][service.subtext] = service
            
            if (not trainers[service.name]) then trainers[service.name] = {} end
            if (not trainers[service.name][service.subtext]) then trainers[service.name][service.subtext] = {} end
            if (not trainers[service.name][service.subtext][npc.name]) then trainers[service.name][service.subtext][npc.name] = npc end
        end
    end
    
    if SchoolNotes and self.db.profile.takeNotes then
        if (not notes.trainers) then notes.trainers = {} end
        if (not notes.trainers[npc.name]) then notes.trainers[npc.name] = {} end
        if (not notes.trainers[npc.name].noted) then 
            self:NoteIt(npc) 
            notes.trainers[npc.name].noted = true
        end
	end

    self.db.char.spells = spells
    self.db.char.trainers = trainers
    self.db.char.notes = notes
end

function SchoolTeacher:LogClassMerchantInfo()
    self:Debug("LogClassMerchantInfo")
    local spells = {}
    local trainers = {}
    local notes = {}

    if (self.db.char.spells ~= nil) then spells = self.db.char.spells end
    if (self.db.char.trainers ~= nil) then trainers = self.db.char.trainers end
    if (self.db.char.notes ~= nil) then notes = self.db.char.notes end
    
    local npc = {}
    npc.name = self:GetTeacherName()
    npc.zone = self:GetTeacherZone()
    npc.subzone = self:GetTeacherSubZone()
    npc.type = self:GetTeacherType()
    
    if (string.lower(npc.type) == "demon trainer") then
        local category = "Pet"
        local numItems = GetMerchantNumItems()
        
        for i = 1, GetMerchantNumItems() do
            local service = {}
            service.cost = {}
            service.fullname, service.icon, service.cost.money, service.quantity, _,_ = GetMerchantItemInfo(i)
            
            if (service.fullname ~= nil) then
                service.type = "unavailable"
                if (category ~= nil) then service.category = category end

                local fa, fx, subtext = string.find(service.fullname, "(Rank %d+)")
                service.subtext = subtext or "None"
                
                if (not fa) then service.name = service.fullname 
                else service.name = string.sub(service.fullname, 1, (fa - 2)) end
                
                local fa, fx = string.find(service.fullname, service.subtext)
                
                tooltip:SetMerchantItem(i)
                local _,_, match = tooltip:Find("(Already known)", 2, 20)
                if (match ~= nil) then service.known = true end
                
                local _,_, level = tooltip:Find("Requires Level (%d+)", 2, 20)
                if (level ~= nil) then service.level = tonumber(level) end
                
                if (service.known) then service.type = "used" end
                
                if (not spells[service.category]) then spells[service.category] = {} end
                if (not spells[service.category][service.name]) then spells[service.category][service.name] = {} end
                if (not spells[service.category][service.name][service.subtext]) then spells[service.category][service.name][service.subtext] = {} end
                spells[service.category][service.name][service.subtext] = service
            
                if (not trainers[service.name]) then trainers[service.name] = {} end
                if (not trainers[service.name][service.subtext]) then trainers[service.name][service.subtext] = {} end
                if (not trainers[service.name][service.subtext][npc.name]) then trainers[service.name][service.subtext][npc.name] = npc end
            end
        
        end
    end
    
    if SchoolNotes and self.db.profile.takeNotes then
        if (not notes.trainers) then notes.trainers = {} end
        if (not notes.trainers[npc.name]) then notes.trainers[npc.name] = {} end
        if (not notes.trainers[npc.name].noted) then 
            self:NoteIt(npc) 
            notes.trainers[npc.name].noted = true
        end
	end

    self.db.char.spells = spells
    self.db.char.trainers = trainers
    self.db.char.notes = notes    
end


function SchoolTeacher:GetTeacherName()
    return UnitName("npc")
end

function SchoolTeacher:GetTeacherZone(teacher)
    return GetZoneText()
end

function SchoolTeacher:GetTeacherSubZone(teacher)
    return GetSubZoneText()
end

function SchoolTeacher:GetTeacherType(teacher)
    local result 
    
    local tooltip = AceLibrary("Gratuity-2.0")
        
    tooltip:SetUnit("npc")
    result = tooltip:GetLine(2)
    
    if (not result) then
        if (not teacher) then teacher = self:GetTeacherName() end
    
        local teacherTypes = {}
        teacherTypes["Aldrae"] = "Priest Trainer"
        teacherTypes["Lotheolan"] = "Priest Trainer"
        teacherTypes["Belestra"] = "Priest Trainer"
        teacherTypes["Ponaris"] = "Priest Trainer"
        teacherTypes["Daestra"] = "Demon Trainer"
        
        result = teacherTypes[teacher]
    end

    -- self:Debug("GetTeacherType:result ==> %s", result)
    return result
end    

function SchoolTeacher:NoteIt(trainer)
    if (not trainer) then return end
    if (not SchoolNotes) then return end
    local note = {}
    note.title = trainer.name
    note.info = trainer.type or ""
    note.creator = "School"
    note.icon = "Interface\\Icons\\INV_Misc_Book_08"
    
    SchoolNotes:SetNote(note)
end

function SchoolTeacher:NoteIt1(trainer)
    self:Debug("NoteIt")
    SetMapToCurrentZone()

    local note = {}
    note.zone = GetRealZoneText()
    note.x, note.y = GetPlayerMapPosition("player")
    note.title = trainer.name
    note.titleR = 0
    note.titleG = 1
    note.titleB = 0
    note.info = trainer.type or ""
    note.infoR = 1
    note.infoG = 1
    note.infoB = 1
    note.info2 = ""
    note.info2R = 1
    note.info2G = 1
    note.info2B = 1
    note.creator = "School"
    note.icon = "Interface\\Icons\\INV_Misc_Book_08"

    -- Cartographer_Notes:SetNote(zone, x, y, 
    --                            title, titleR, titleG, titleB, 
    --                            info, infoR, infoG, infoB, 
    --                            info2, info2R, info2G, info2B, 
    --                            creator, icon, manual, oldId)
    
    Cartographer_Notes:SetNote(
                               note.zone, note.x, note.y,
                               note.title, note.titleR, note.titleG, note.titleB,
                               note.info, note.infoR, note.infoG, note.infoB,
                               note.info2, note.info2R, note.info2G, note.info2B,
                               note.creator, note.icon, note.manual, note.oldId
    )

end
